<?php
/*
        Author: Alex Nosurov
        Date: 05.02.2018
        Description: reads settings, alerts (from table) and performs alert action (send mail) or not depending on settings
	Requirements:
		PHPMailer 5.2
	run by:
		process_alerts(); // not catches db errors, only catches single receiver send error
		php alerts.php run  // from cmd line

        Input: this version takes data from alerts table (which naturally will fail if having db problems)
                file of settings in same dir with section 'alerts'
			[alerts]
			log_file = 'logs/alerts.log'  // def=alerts.log
			gravity_min	// alerts with smaller gravity not processed [1..5], def = 3
			max_alerts_count // def=50, if num alerts processed at one launch exceeds this - rest of alerts not sent (but set as processed), instead is sent one msg saying that limit exceeded
			sender_email = 'mail@alogica.it' // obligatory
			sender_name = 'alert manager'
			receiver_email = 'alex.nosurov@alogica.it' // obligatory sender1;sender2
			email_template   // def = 'html/alert.html'
			images_dir = 'images/' // path for images of the email
			// authentication on mail server
			host = 'hetzner'
			smtp_auth = 1 // def=0
			smtp_host  // def = localhost; e.g.: 'mail.alogica.it'
			smtp_post = 25
			smtp_user = 'uuu'
			smtp_password = 'xxx'
	Also need:
		alert mail template: 
		set by param email_template, def = html/alert.html
		template may use images, each must be embedded;
		the PHPMailer library parses html and for each img found calls embed function
		the code replaces 'alert_img' tag (if any) with img_dir/alertX.png where X - is alert level; that may be used to insert image showing alert level
	authentication:
		mail sending may need authentication:
		smtp_host, smtp_port, smtp_user, smtp_password (settings.ini)

        Output: alert action (send mail)

	Database:
	db (mysql version):
	CREATE TABLE `alerts` (
	  `id` int(11) NOT NULL AUTO_INCREMENT,
	  `message` text,
	  `module` varchar(20) DEFAULT '',
	  `gravity` int(11) DEFAULT '3',
	  `type` varchar(20) DEFAULT 'unknown',
	  `date_alert` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
	  `sent` int DEFAULT '0',
	  `scan` int DEFAULT '0',
	  PRIMARY KEY (`id`)

	psql version:
	create table alerts(
	 id serial PRIMARY KEY,
	 message text,
	 module varchar(20),
	 gravity int default 3,
	type varchar(20) default 'unknown',
	 date_alert timestamp default now(),
	 sent int default 0,
	 scan int default 0
	);


	DEF POSITIVE_ALERT: need differentiate gravity to importance + positive/negative, so that to be able to find important but positive alerts
	that may be done by: a) add a flag pos/neg b) use 6-9 for positive
	db: convenient to have flag+importance to sort/filter; in funcs and text-err-file better not add another param, so use 6-9. 
	so use 6-9 and use view for db:  gravity mod 5 as gravity, if(gravity>5,1,0) as positive

*/
	require_once(dirname(__FILE__)."/path.php");
	require_once(proj_path."utils/utils.php");
	require_once(proj_path."utils/db_mysql.php");
	require_once(proj_path."model/block.php");
	//require_once(dirname(__FILE__)."/../PHPMailer5.2.14/PHPMailerAutoload.php");
	//require_once(lib_path."PHPMailer/PHPMailerAutoload.php");
	require_once(proj_path."utils/php_mailer_include.php");
	require_once(proj_path."utils/log.php"); // added it 21.09.22; err in php_errors.log: restore_log_file_name not found


	// TODO: why it's not errors.php???
	define('el_none' , 0); // just an important msg to inform
	define('el_low' , 1);
	define('el_medium' , 2);
	define('el_high' , 3);
	define('el_higher' , 4);
	define('el_highest' , 5);
	// positive alerts POSITIVE_ALERT
	define('il_low' , 6);
	define('il_medium' , 7);
	define('il_high' , 8);
	define('il_higher' , 9);
	define('il_highest' , 10);
	function al_get_imp($g){return $g<6? $g : $g-5;} // get importance

	// gravities defined here (el_none ...), types are in errors.php (but may use any string)
        function write_alert($msg, $module='', $gravity=3, $type=''){
                $date = date( db\get_date_format() );
		if(!db\is_db_dos_error($msg)){
			try{
				db\execute_to("insert into alerts(message,module,gravity,type,date_alert) values('".db\escape($msg)."', '$module', $gravity, '$type', '$date')");
			}catch(Exception $e){
				log\set_log_file(unregistered_log_file);
				log\error("error inserting alert: ".$e->getMessage()."; ".$msg);
				restore_log_file_name();
			}
		}else{
			log\set_log_file(unregistered_log_file);
			log\error("non inserting to db while is a db dos error: '$msg'");
			restore_log_file_name();
		}
        }
	// format of an alert: message,module,gravity,type,date
        function read_alerts(){
                // need know which recs were read
                db\execute("update alerts set scan=1 where sent=0");
                // select only read messages
                return db\select_to_array("alerts", "*", "sent=0 and scan=1", "date_alert asc");
        }
	function read_alerts_file(){
                $settings = parse_ini_file(dirname(__FILE__).'/alerts_file.ini', false);  // conains lines; line: num "message"; errs in cpp err file contain a num code; alerts_file.ini contains msgs for each code
		$alert_file = cpp_error_file;
		log\info("reading alerts from file: $alert_file");
		$res = array();
		if( !file_exists($alert_file) ){
			log\info("file not found");
			return $res;
		}
		$id = 0; // instead of id (as in db) use just line num
		$alerts = file($alert_file);
		$regex = "/(\w+) (\d+) (\d+) \"([^\"]+)\" \"([^\"]*)\"/";
		foreach($alerts as $alert){
			$matches = array();
			$alert_db = array(); // alert in db format
			if( preg_match( $regex, $alert, $matches)==1 ){
				$alert_db['module'] = $matches[1];
				$alert_db['gravity'] = $matches[2] + 1;
				$alert_db['type'] = get_or_def( $settings, $matches[3], "alert");
				$alert_db['date_alert'] = $matches[4];
				$alert_db['message'] = $matches[5];
				$alert_db['id'] = ++$id;
			}
			$res[] = $alert_db;
		}
		return $res;
	}
        function flush_alerts(){
                db\execute("update alerts set sent=1 where scan=1"); // actually for the DB may flush messages individually
        }
	function flush_alerts_file(){
		// alerts that are already read are in alerts.log.read file; this code appends alert.log to curr alerts.log.read and then empties alert.log
		$alert_file = cpp_error_file;
		$pi = pathinfo($alert_file);
		if( file_exists($alert_file) && filesize($alert_file) ){
			//exec_log("mv $alert_file " . $pi['dirname'] . "/" . $pi['filename'] . "_" . date("Y_m_d_H_i_s", filemtime($alert_file) ) . "." . $pi['extension'] ); // thus have many little files; instead, add contents to some file like alert.log.read
			$prev_read = file_exists($ar=$alert_file.".read")? $ar : "";
			exec_log("cat $prev_read $alert_file > $alert_file.tmp"); // concat curr alert file and prev alert.read to tmp (can't use alert.read both as src and dest, so use tmp)
			exec_log("mv -f $alert_file.tmp $alert_file.read"); // TOTEST
			exec_log("rm -f $alert_file"); // TOTEST
		}
	}
	// TODO: not used now; PHPMailer inserts images itself
        function addImage($mail, $template, $tag, $image, $id){
                $imgp = dirname(__FILE__)."/img/$image";
                $mail->AddEmbeddedImage($imgp, $id, $image, 'base64', 'image/png'); // the only image need - gravity
                return replace_keys($template, array($tag => $imgp) );
        }
        function alert_action_mail($alert, $settings_alerts, $count=1, $attachments=NULL){
                $sender = must_be_set($settings_alerts, 'sender_email');
                $receiver = must_be_set($settings_alerts, 'receiver_email');
		if( $alert['gravity']>=el_higher && ($admin_email=get_or_def($settings_alerts, 'receiver_email_admin')) )
			$receiver.= ";$admin_email";
		$template = get_or_def($settings_alerts, 'email_template', dirname(__FILE__).'/html/alert.html');
		$img_path = get_or_def($settings_alerts, 'images_dir', dirname(__FILE__)."/img");

				$mail = new PHPMailer\PHPMailer\PHPMailer();
                //$mail = new PHPMailer();
                $mail->IsSMTP();                                      // set mailer to use SMTP
                $mail->Host = get_or_def($settings_alerts, 'smtp_host', "localhost");  // specify main and backup server
                $mail->Port = get_or_def($settings_alerts, 'smtp_port', "25");  
		if( get_or_def($settings_alerts, 'smtp_auth', 0) ){
			$mail->SMTPAuth = true;
			$mail->Username  = get_or_def($settings_alerts, 'smtp_user');  
			$mail->Password = get_or_def($settings_alerts, 'smtp_password');  
		}
		$mail->From = $sender;
		$mail->FromName = get_or_def($settings_alerts, 'sender_name', '');
                $emails = explode(";",$receiver);
		//var_dump($emails);
		echo "\n'$count'\n";
                foreach($emails as $email)
                        $mail->AddAddress($email);               // name is optional

                $mail->WordWrap = 50;                                 // set word wrap to 50 characters
                $mail->Subject = $alert['subject'];
		$message = "";
		if($template && file_exists($template) ){
			$mail->IsHTML(true);                                  // set email format to HTML
			$message = file_get_contents($template);
			$alert['alert_img'] = "alert" . $alert['gravity'] .".png";
			$alert['count'] = $count;
			// $message = addImage($mail, $message, "logo_client", "logo_client.png", 'lc');  no neeed of this - it's done by msgHTML()
			$message = replace_keys($message, array_merge($settings_alerts, $alert) );
			$mail->msgHTML($message, $img_path); 
			//var_dump($message);
		}
		else{
			$message = $alert['message'];
			$mail->Body = $message; 
		}
                $mail->AltBody = "Error in module {$alert['module']}: {$alert['message']}"; // this is for bros not supporting HTML
		// new 19.01.24 - attachments
		if($attachments)
			foreach($attachments as $at)
				$mail->addAttachment($at);
		//
		if(! $mail->send() ){ 
			//var_dump($mail);
			throw new Exception("PHPMailer send error (check credentials) ".$mail->ErrorInfo);
		}
                unset($mail);
        }
	function init_alerts(){
		// set log and get settings
		$settings = get_settings();
		/*$log_file = get_or_def2($settings, 'alerts', 'log_file', 'alerts.log');
		log\set_log_file($log_file); // log at least to the default log      commented: when called from controller - replaces log file and not returns it   */
		if( !isset( $settings['alerts'] ) )
				throw new Exception("Section alerts not defined in the file of settings!");
		return must_be_set($settings, 'alerts');
	}
	// next is new, added 19.01.24 to send a single alert with attachment without putting it to DB; TODO: maybe would be easier to add field "attachments" to db
	function send_alert($msg, $subject, $gravity, $attach){
                $settings_alerts = init_alerts();
		alert_action_mail( array("message" => $msg, "subject" => $subject, "gravity" => $gravity), $settings_alerts, 1, $attach); // TODO: date?
	}

        function process_alerts($source="DB", $admin=false){ // DEF ADM_AL cpp alerts should be sent only to admin (usvers get mad); so may pass $admin=true to sent alerts only to admin; related with IMP_ERR (cpp)
	try{
                $settings_alerts = init_alerts();
		// to avoid text alerts being processed twice; TODO: but may happen that in case of error block flag remains; maybe put to block files pids and delete those with not-existing pids;
		//NOTE: that function is called twice at same time - to process text/db alerts - it causes flag conflict uless add wait flag time
		block_flag("alerts", 30); 
		// set log and get settings
		$settings = get_settings();
		$log_file = get_or_def2($settings, 'alerts', 'log_file', 'alerts.log');
		log\set_log_file($log_file); // log at least to the default log
		log\set_log_level(get_or_def2($settings, 'alerts', 'log_level', 'info')); // log at least to the default log
		if( !isset( $settings['alerts'] ) )
				throw new Exception("Section alerts not defined in the file of settings!");
		log\info("Starting alerts");
		$settings_alerts = array();
		$settings_alerts = must_be_set($settings, 'alerts');
		$gravity_min = get_or_def($settings_alerts, "gravity_min", 3); // 1..5
		$max_alerts_count = get_or_def($settings_alerts, "max_alerts_count", 50);
		if($admin) // ADM_AL
			$settings_alerts["receiver_email"] = $settings_alerts["admin_email"];

                // read alerts (not yet processed)
                // ass array: date, module, type, gravity, message
		if($source=="DB")	$alerts = read_alerts(); // this function is changed according to the method of storing alerts; now it's db but may use a text file;
		if($source=="FILE")	$alerts = read_alerts_file(); // this function is changed according to the method of storing alerts; now it's db but may use a text file;
		if($source=="UNREG")	$alerts = read_alerts_unregistered(); // this function is changed according to the method of storing alerts; now it's db but may use a text file;
                log\info("Alerts found: ".sizeof($alerts) );
		//var_dump($alerts); return;

                // iterate all events (not processed)
                $alerts_sent_num = 0;
		$same_msg_count = 1;
		$max_index = sizeof($alerts) - 1;
		$msg_prev = "";
                foreach($alerts as $key => $alert){
                        // control over number of alerts sent at one launch; NOTE: shouldn't launch alert manager too often to avoid flooding emails!
                        if( $alerts_sent_num >= $max_alerts_count ){
                                $subj = lp("alert_many")." (".sizeof($alerts)."); ";
                                log\error($subj);
                                alert_action_mail( array("subject" => $subj, "message" => $subj.lp("alert_read"), "date_alert" => date("y/m/d h:i"), "module" => "alerts", "type" => "alerts", "gravity" => 5), $settings_alerts); // NOTE: not normal alert - summary
                                break;
                        }
                        $msg = $alert['message'];
                        $id= $alert['id'];
                        if( al_get_imp($alert['gravity']) < $gravity_min ){
                                log\info("message $id skipped by gravity threshold");
                                continue;
                        }
			// in the group of equal messages only the last is sent; check if this is not group's middle
			if($key<$max_index && $msg==$alerts[$key+1]['message']){
				log\info("not sending while next alert is the same");
				$same_msg_count++;
				continue;
			}
                        // process the alert normally
			try{
				$alert['subject'] = "Alert system ({$settings_alerts['host']})" . ": " . $alert['type'];
				alert_action_mail($alert, $settings_alerts, $same_msg_count);
			} catch (phpmailerException $e) {
				log\error("Send mail error: ".$e->errorMessage()); 
			} catch (Exception $e) {
				log\error("Send mail error: ".$e->getMessage());
			}
                        log\info("Alert $id sent");
                        $alerts_sent_num++;
			$msg_prev = $msg; // remember only sent messages
			// reset msg group
			$same_msg_count = 1;
                }
                //
		if($source=="DB")	flush_alerts(); // this function is changed according to the method of storing alerts; now it's db but may use a text file;
		if($source=="FILE")	flush_alerts_file(); // this function is changed according to the method of storing alerts; 
		if($source=="UNREG")	flush_alerts_unregistered(); // 
		//
		unblock_flag("alerts");
                log\info("Finished alerts");
	}catch(Exception $e){
		unblock_flag("alerts"); // anyway need unblock this flag; once happend that mysql gave error (gone away) which caused flag block
		throw $e;
	}
        }

	function read_alerts_unregistered(){
		$alerts = array();
		if(file_exists(unregistered_log_file))
			$alerts[] = array( "message" => file_get_contents(unregistered_log_file), "id" => 0, "gravity" => el_highest, "subject" => "unregistered alerts", "type" => "alerts", "date_alert" => date('d-m-Y H:i:s'), "module" => "alerts" );
		return $alerts;
	}
	function flush_alerts_unregistered(){ if(file_exists(unregistered_log_file)) exec_log("mv ".unregistered_log_file." ".unregistered_log_file."1"); }


        ////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////

	if(isset($argc) && $argc>1 && $argv[1]=='run_alerts')
                process_alerts();

        ////////////////////////////////////////////////////////////////////////////////////////
        ////////////////////////////////////////////////////////////////////////////////////////
?>

