<?php
/*
	Created: 20.02.2018
	Author: Alex Nosurov
	Description: db model functions for the archive
*/
	require_once(dirname(__FILE__)."/../path.php");
	require_once(proj_path."model/db.php");
	require_once(proj_path."model/email.php");


	// get users related with a shop: visual, production internal, graphic, production
	function get_shop_users_id($shop_id, $role){ 
		if($role=="LOCAL_VISUAL") 		return array(db\select_field("shops_groups", "visual_id", "id=$shop_id"));
		if($role=="PRODUCTION")return array(db\select_field("shops_groups", "production_id", "id=$shop_id"));
		//return  db\select_field_to_array("users", "id", "concat(user_role,';') like '%$role;%'");
		return  db\select_field_to_array("users", "id", db\in_list("user_role", $role));
	}
	function open_camp($camp_id){
		$camp_name = db\select_field("camps", "name", "id=$camp_id");
		$event_id = event_create(db\escape2("Campaign started: $camp_name"), $camp_id, 0, 0, page_link($camp_id, 0, "campaigns"));
			$users = db\select_to_array("users", "email, id, user_role", "user_role<>'' and user_role is not null", "user_type desc"); // note: trick with order just to put Users before Groups
		$camp_shops = get_camp_shops($camp_id); 
		foreach( $users as $user){
			$user_roles = preg_split('/;/', $user["user_role"], -1, PREG_SPLIT_NO_EMPTY); // now users.user_role is a ';' separated string list of roles
			foreach($user_roles as $user_role){
				if($user_role=="PRODUCTION" || $user_role=="LOCAL_VISUAL"){
					$user_shops = get_user_shops($user["id"], $user_role);
					if( !sizeof(array_intersect($camp_shops, $user_shops)) ) continue; // skip users which shop list not intersect with camp shops
				}
				$page = in_array($user_role, array("LOCAL_VISUAL", "PRODUCTION", "VISUAL_ADMINISTRATOR"))? "shops" : "campaigns";
				add_event_user($event_id, $user["id"], page_link($camp_id, 0, $page, $user_role, $user["id"] ));
			}
		}
		email_camp_start($event_id, $camp_id); // EMAIL
		db\execute("update camps set camp_state='opened' where id='$camp_id'"); 
	}
	// ORDERS
	// created by visual, informed to prod int
	function create_order($fields, $user_id){
		// check if already exists
		$camp_id = $fields["camp_id"]; $shop_id = $fields["shop_id"]; 
		if(db\exists("orders", "shop_id=$shop_id and camp_id=$camp_id")){log\error("order for $camp_id/$shop_id already exists");return 0;}
		if(!db\exists("v_lbs", "shop_id=$shop_id and camp_id=$camp_id")){log\error("no lbs for $camp_id/$shop_id");return 0;} // avoid create empty order
		//
		log\info("creating order for $camp_id/$shop_id");
		$fields["order_user_id"] = $user_id; 
		db\execute("insert into orders".db\compile_insert($fields));
		$order_id = db\get_last_id();
		// create event EVENTS
		$link = page_link($camp_id, $order_id, "orders");
		$event_id = event_create("New order created", $camp_id, $shop_id, $order_id, $link);
		foreach( get_shop_users_id($shop_id, "PRODUCTION_INTERNAL") as $user_id){
			$link = page_link($camp_id, $order_id, "orders", "PRODUCTION_INTERNAL", $user_id );
			add_event_user($event_id, $user_id, $link);
		}
		//
		send_order_email($event_id, $order_id);
		return $order_id;
	}
	function delete_order_email($order_id){
		$order = db\select_record("v_orders", "*", "id=$order_id");
		$camp_id = $order["camp_id"]; $shop_id = $order["shop_id"]; 
		// create event EVENTS
		$link = page_link($camp_id, 0, "orders");
		$event_id = event_create("Order deleted", $camp_id, $shop_id, $order_id, $link);
		$users = db\select_field_to_array("users", "id", db\in_list("user_role", "ADMIN")." OR ".db\in_list("user_role", "ENGINEERING")." OR ".db\in_list("user_role", "PRODUCTION_INTERNAL") );
		foreach( $users as $user_id)
			add_event_user($event_id, $user_id, $link);
		$event = db\select_record("events", "*", "id=$event_id");
		$subj = "Order $order_id deleted: ".$order["shop_name"];
		$order["user_name"] = db\select_field("users", "name", "id=".get_curr_user_id() );
		$order["camp_name"] = db\select_field("camps", "name", "id=$camp_id" );
		order_delete($order_id);
		send_user_emails($event_id, $order, $subj, "order_del", "");
	}
	// TODO: maybe better cache order_id in lbs table (simpler)
	function get_lbs_order($camp_id, $shop_id){ 
		return $order = db\select_record("v_orders_state1", "*", "camp_id=$camp_id and shop_id=$shop_id");
	}

	function reject_lbs($app_id, $receivers, $subj, $msg){
		$lbs = db\select_record("app_lbs", "id, shop_id, camp_id, app_state, name", "app_id=$app_id");
		$camp_id = $lbs["camp_id"]; $shop_id = $lbs["shop_id"];
		if( $lbs["app_state"]=="rejected" ) return;
		db\execute("update lbs set app_state='rejected' where id = $app_id");
		$order = get_lbs_order($camp_id, $shop_id);
		$order_id = $order["id"];
		// create event EVENTS
		$link = page_link($camp_id, $order_id, "orders");
		$event_id = event_create($subj, $camp_id, $shop_id, $order_id, $link);
		$link = page_link($camp_id, $order_id, "orders", "PRODUCTION_INTERNAL" );
		foreach( get_shop_users_id($shop_id, "PRODUCTION_INTERNAL") as $user_id){
			$link = 	 page_link($camp_id, $order_id, "orders", "PRODUCTION_INTERNAL", $user_id );
			$link_apap = page_link($camp_id, $app_id  , "apap", "PRODUCTION_INTERNAL", $user_id);
			add_event_user($event_id, $user_id, $link, $link_apap);
		}
		//
		send_order_email($event_id, $order_id, $app_id, $msg, array("subj_name" => $lbs["name"]) ); // TODO: app_id or lbs_id?
		// remove old files
		$tp = get_source_path().get_triple_path($camp_id, $app_id);
		foreach(scandir($tp) as $file)
			if(extract_ext($file)=="idml" || extract_ext($file)=="indd")
				exec_cmd("rm", $tp.$file);
	}
	// role seems is the dest role which is to receive the notif (eg order uploaded email goes to graphic); 
	function set_lbs_state($id, $state, $message, $role){
		$lbs = db\select_record("app_lbs", "order_id, app_state", "id=$id");
		if( $lbs["app_state"]==$state ) return;
		db\execute("update lbs set app_state='$state' where id=$id");
		// check if order printed completely;
		$order_id = $lbs["order_id"];
		$order = db\select_record("v_orders_state","*","id=$order_id"); // TODO: use app_lbs.order_id here and in other places instead
		$camp_id = $order["camp_id"]; $shop_id = $order["shop_id"];
		if($order && $order[$state]){ // NOTE: this means lbs state should have same name that order state (printed/approved)
			// create event EVENTS
			$link = page_link($camp_id, $order_id, "orders");
			$event_id = event_create($message, $camp_id, $shop_id, $order_id, $link);
			foreach( get_shop_users_id($shop_id, $role) as $user_id){
				$link = page_link($camp_id, $order_id, "orders", $role, $user_id); $tlink="";
				if($role=="PRODUCTION")
					$tlink = "contr/download.php?link=".create_link($order_id, "triples", $user_id)."&portal=".get_portal_name();
				add_event_user($event_id, $user_id, $link, "", $tlink );
			}
			$data = array();
			if($role=="PRODUCTION")
				//$data["hurl"] = base_url.proj_path_rel."download.php?link=".create_link($camp_id, "hires_zip")."&portal=".get_portal_name();
				$data["hurl"] = base_url.proj_path_rel."hires.php?portal=".get_portal_name();
			//
			send_order_email($event_id, $order_id, 0, '', $data);
		}
		return $order; // just used in print_order to avoid re-find order info
	}
	function print_lbs($id){ 
		$order = set_lbs_state($id, "printed", "Order completed", "ENGINEERING"); 
		if($order && $order["printed"])
			db\execute("update orders set date_done='".db\now()."' where date_done is null and id = ".$order["id"]); // // set order done date
	}

	// DEF EVENTS register events
	function event_create($message, $camp_id, $shop_id=0, $order_id=0, $source_order_link=""){
		$ugr = get_curr_group_id(); $uid = get_curr_user_id();
		if(!$uid) $uid = "0"; // TODO: happens in EMAIL_LINK, need yet to pass somehow user who downloads
		db\execute("insert into events (source_group_id, source_user_id, message, camp_id, shop_id, order_id, source_order_link) values ($ugr, $uid, '$message', $camp_id, $shop_id, $order_id, '$source_order_link')");
		return db\get_last_id();
	}
	// those are users-dests TODO: always need only either link, maybe use single field for them both
	function add_event_user($event_id, $user_id, $link, $link_apap="", $link_triple=""){
		db\execute("insert into event_users(event_id, dest_group_id, order_link, apap_link, triple_link) values ($event_id, $user_id, '$link', '$link_apap', '$link_triple')");
	}

	/* called by cron, closes all expired */
	function close_camps(){
		$now = db\now();
		$ctc = db\select_to_array("camps", "id,name", "camp_state in ('opened','published') and date_till is not null and date_till<'$now'"); 
		foreach($ctc as $camp)
			close_camp($camp["id"]);
	}
	function close_camp($camp_id){
		db\execute("update camps set camp_state = 'closed' where id = $camp_id");
		$event_id = event_create("Campaign closed", $camp_id);
		foreach( get_admin_users() as $user)
			add_event_user($event_id, $user["id"], page_link($camp_id, $rec_id=0, $page="campaigns", $role=$user["user_role"], $user["id"]) );
	}

	// called by cron ARCH
	function archive_camps(){
		$cta = db\select_to_array("camps", "id,name", "camp_state in ('closed')");
		foreach($cta as $camp){
			// TODO: delete images
			db\execute("update camps set camp_state = 'archived' where id = ".$camp["id"]);
			$event_id = event_create("Campaign archived", $camp["id"]);
			// archive lbs
			db\execute("insert into app_lbs_arch select * from app_lbs where camp_id = ".$camp["id"]);
			db\execute("insert into v_orders_state1_arch select * from v_orders_state1 where camp_id = ".$camp["id"]);
			db\execute("delete from approves where camp_id = ".$camp["id"]);
			db\execute("delete from lbs where camp_id = ".$camp["id"]); // DELBS
			db\execute("delete from orders where camp_id = ".$camp["id"]);
			// notify
			foreach( get_admin_users() as $user)
				add_event_user($event_id, $user["id"], page_link($camp["id"], $rec_id=0, $page="campaigns", $role=$user["user_role"], $user["id"]) );
		}
	}
	
?>
