<?php
class SHM {
	private static $SEMAPHORE_KEY = 291623582;                     //Semaphore unique key (MAKE DIFFERENT TO RPi App KEY) 
	private static $SHARED_MEMORY_KEY = 672213396;         //Shared memory unique key (SAME AS RPi App  KEY) TODO: normally obtained by ftok(path,proj)
	// memory structure
	private static $control_byte = 0;// byte which contains control code
	private static $flag_byte = 1;// now used for refresh request
	    // out
	private static $table_byte = 2; // the byte which contains the table num to be used
	private static $path_byte = 3; // the byte which contains path_id
	    // in
	private static $c_data_byte = 2;  // control code
	// control states
	private static $c_ready = 1;  
	private static $php_data = 3; // control code
	private static $c_data = 4;  // control code
	private static $php_done = 6; // control code
	private static $exit = 8; // control code
	private static $c_error = 9;  // control code
	private static $ping = 10;  // control code
	// vars
	private $shm_id;
	private $shm_len;
	private $current_state;

	// time given (seems) in 1/10 sec
	// state may be scalar (one state) or array (several states)
	// TODO: need mutex during which shm is captured/released
	function wait_state($state, $time, $stop_on_error = false){
		$this->current_state = "";
		while($time--){
			$shared_memory_string = shmop_read($this->shm_id, self::$control_byte, 1);                           //Shared memory ID, Start Index, Number of bytes to read
			if($shared_memory_string == FALSE)
				throw new Exception("Failed to read shared memory");  
			$shared_memory_array = array_slice(unpack('C*', "\0".$shared_memory_string), 1); 
			$this->current_state = $shared_memory_array[0];
			if(!is_array($state) && $state==$this->current_state) 
				return true;
			if(is_array($state) && in_array($this->current_state, $state) )  // several flags were passed (need stop when at least one present
				return true;
			if($stop_on_error && $this->current_state==self::$c_error)
				return false;
			usleep(100000);
		}
		return false;
	}
	function read_data(){
		return trim(shmop_read($this->shm_id, self::$c_data_byte, $this->shm_len - self::$c_data_byte));    //Shared memory ID, Start Index, Number of bytes to read
	}

	function write_byte($byte, $pos=0){
		shmop_write($this->shm_id, pack("C1", $byte), $pos );
	}
	function write_str($str, $pos){
		$str_len = shmop_write($this->shm_id, $str, $pos); // write the string to shm 
		$this->write_byte(0, $pos0 = $pos + $str_len ); // put trailing 0 byte
		log\debug("p: $pos, w: $str");
		return $pos0+1; // the next byte to write
	}

	public function __construct(){
		$this->shm_id = shmop_open(self::$SHARED_MEMORY_KEY, "w", 0, 0);  //Shared memory key, flags, permissions, size (permissions & size are 0 to open an existing memory segment)
		if (empty($this->shm_id))
			throw new Exception("Failed to open shared memory");  // c not runs
		$this->shm_len = shmop_size($this->shm_id);
	}
	public function __destruct(){
		if (empty($this->shm_id))
			shmop_close($this->shm_id);//Detach from the shared memory
	}
	// used for both open and find operations
	public function open_dir($table_num, $path, $find=false){
		// WAIT C READY
		if(!$this->wait_state(self::$c_ready, 50))
			throw new Exception("C is not ready");

		// DATA FOR C   TODO: while preparing data - some other proc may capture shm, so here need some mutex or at least need quickly denote that shm is busy (data not yet ready but shm busy)
		if(!$find) // normal mode - used for open dir action
			$this->write_str($path, self::$path_byte);
		else{ // this is for find DEF FINDSHM - find with shm; in this case the struct of data is: $$$find <params>   param: name0op0value0
			$pos = self::$path_byte;
			$pos = $this->write_str('$$$find', $pos);
			$debug_msg = "";
			foreach($path as $elem) // elem is a single filter condition: path,like,bla
				foreach($elem as $word){
					$pos = $this->write_str($word, $pos); // TODO: control max len: buff size=buff_len, managed in cpp; so cpp needs inform it writing it to a special byte or may put it statically to settings; then write_str() should control it;
					$debug_msg.=" ".$word;
				}
			log\info("Find request: table: $table_num; request: $debug_msg");
			$pos = $this->write_str('$$$find', $pos);
		}

		$this->write_byte($table_num, 	   self::$table_byte);
		$this->write_byte(self::$php_data, self::$control_byte);

		// WAIT C RESULT
		if(!$this->wait_state( array( self::$c_data, self::$c_error) , 50, true)) // wait untill res ok  or  error
			if($this->current_state==self::$c_error){
				$this->write_byte(self::$php_done);
				throw new Exception("C error (too big result?)");
			}
			else
				throw new Exception("no C response");

		// PROCESS C RESULT
		$res = $this->read_data();
		log\debug("find res raw: $res");
		//var_dump($res);
		$paths_files = explode('$$$', $res);
		$paths = array_diff( explode(' ', $paths_files[0]), array("") );
		$files = array_diff( explode(' ', $paths_files[1]), array("") );

		// SAY TO C DATA IS READ
		$this->write_byte(self::$php_done);

		return array("paths" => $paths, "files" => $files);
	}
	//
	function refresh_tables($find_new_paths=false){ // inform c that it's necessary to refresh tables
		$this->write_byte($find_new_paths?2:1, self::$flag_byte);
	}
	public function ping($wait_sec=5){
		$wsd = $wait_sec*10; // how many 1/10 secs to wait
		if(!$this->wait_state(self::$c_ready, $wsd))
			return false;
		$this->write_byte(self::$ping);
		if(!$this->wait_state(self::$c_ready, $wsd))
			return false;
		return true;
	}
	public function request_exit(){ $this->write_byte(self::$exit); }

}
function ping_shm(){
	log\debug("ping shm...");
	$res = false;
	try{
		$shm = new SHM(); // actually only this throws
		log\debug("shm created");
		$res = $shm->ping();
		log\debug("ping: $res");
	}catch(Exception $e){ }
	unset($shm);
	return $res;
}

function exit_shm(){
	$shm = new SHM();
	$shm->request_exit();
	unset($shm);
}
//exit_shm();

//	$shm = new SHM();
//	$res = $shm->refresh_tables();
//	$shm->refresh_tables();
//	var_dump($res);
//	unset($shm);
?>

