<?php
namespace log;
require_once(dirname(__FILE__)."/utils.php");
require_once(dirname(__FILE__)."/../path.php"); // PORTAL get_settings

$log_levels = array("quiet","fatal","error","info","verbose","debug","debug1","debug2","debug3");
$settings = array();
try{
	$settings = get_settings();
}catch(Exception $e){}
$log_level = 		get_or_def2($settings, "general", "log_level", 	"info");
$log_level_prev = $log_level;
$log_file_name = 	get_or_def2($settings, "general", "log_file", 	dirname(__FILE__)."/../messages.log");
$log_file_name_prev = $log_file_name;
$user =		get_or_def2($settings, "general", "user", 	get_current_user() );
$group =	get_or_def2($settings, "general", "group", 	$user );
$mode =		get_or_def2($settings, "general", "mode", 	0775);
$print_user_id = 		get_or_def2($settings, "general", "print_user_id", 	""); // DEF LOG_USER_ID - show user id in log
// a way to put log files into some secondary log; here put messages too frequent, like those when refrshing; is used for log_level>debug3
$log_file_name_secondary = 	get_or_def2($settings, "general", "log_file_secondary",	dirname(__FILE__)."/../secondary.log");
$use_secondary_log = 	get_or_def2($settings, "general", "use_secondary_log",	true);

$stash = false; // to stash messages: sometimes modules called periodically only pollute log; to avoid that module calls stash() - files are written to string, then if need print stashed msg (if call was significant) - call stash(false); now it works so, that when stash is stopped - stashed is shown only when the next msg is written; thus if nothing is written - stash is lost (indeed, normally it's not necessary in that case)
$stashed = ""; // the stashed text
// indent before writing
//_write("\n"); commented while causes new line on each timer from a page
$prefix=""; // used to add some prefix to all log lines

function stash($yes=true, $clear=false){
	global $stash, $stashed;
	$stash=$yes;
	if($clear)
		$stashed="";
}

// returns prev log file
function set_log_file($name){
	global $log_file_name, $log_file_name_prev;
	//$log_file_name_prev = $log_file_name;   keep it always original
	$log_file_name = $name;
}
function restore_log_file_name(){
	global $log_file_name, $log_file_name_prev;
	$log_file_name = $log_file_name_prev;
}

function get_log_file(){
	global $log_file_name;
	return $log_file_name;
}

function set_log_prefix($pref){ global $prefix; $prefix = $pref; }

function write($msg_level, $msg, $msg1='', $msg2='', $msg3='', $msg4='', $msg5='', $msg6=''){
	global $log_levels, $log_level, $use_secondary_log, $log_file_name_secondary, $log_file_name, $print_user_id, $prefix;
	$msg_level_index = array_search($msg_level, $log_levels);
	$level_index = array_search($log_level, $log_levels);
	if($msg_level_index<0)
		$msg_level_index = 3;
	if($level_index<0)
		$level_index = 3;
	$out_of_level = $msg_level_index > $level_index;
	if($out_of_level && !$use_secondary_log)
		_write("$out_of_level");//"return;
	// using secondary log if out of level and writing to secondary log is allowed
	if($out_of_level){
		$_log_file_name = $log_file_name; // save original log file name
		$log_file_name = $log_file_name_secondary; // set to secondary
	}
	$pid = str_pad(getmypid(), 7, "0", STR_PAD_LEFT); // new 1.04.19 print pid
	$sid = $print_user_id? (isset($_SESSION) && isset($_SESSION["user_id"]) ? $_SESSION["user_id"] : "") : "";
	if(is_array($msg)) $msg = dump($msg);
	if(is_array($msg1)) $msg1 = dump($msg1);
	if(is_array($msg2)) $msg2 = dump($msg2);
	if(is_array($msg3)) $msg3 = dump($msg3);
	if(is_array($msg4)) $msg4 = dump($msg4);
	$pref = $prefix? $prefix." " : "";
	_write(date('d-m-Y H:i:s').' '.$pref.$sid.' '.$pid.' '.$msg_level.': '.$msg.' '.$msg1.' '.$msg2.' '.$msg3.' '.$msg4.' '.$msg5.' '.$msg6."\n");
	// restore log file
	if($out_of_level)
		$log_file_name = $_log_file_name;
}
function _write($msg){
	global $log_file_name, $stash, $stashed;
  try{
	if($stash){
		$stashed .= $msg;
		return;
	}
	$new = !file_exists($log_file_name);
	$log_file = fopen($log_file_name, "a+");
	if( !$log_file )
		throw new \Exception("Failed to open log file");
	if(!$stash && $stashed){
		$msg = $stashed.$msg;
		$stashed="";
	}
	fwrite($log_file, $msg); 
	fclose($log_file);
	if($new){
		global $user, $group, $mode;
		chown($log_file_name, "$user");
		chgrp($log_file_name, "$group");
		chmod($log_file_name, $mode);
	}
  }catch(\Exception $e){
	error_log("Failed to write to log file! Log file name: '".$log_file_name.'. Error: '.$e->getMessage(), 
		0); // write to error log TODO: may use mailing (1,3) or SAPI (4) - what's that?
  }
}
function info($msg, $msg1='', $msg2='', $msg3='', $msg4='', $msg5='', $msg6=''){
	write("info", $msg, $msg1, $msg2, $msg3, $msg4, $msg5, $msg6);
}
function error($msg, $msg1='', $msg2='', $msg3='', $msg4='', $msg5='', $msg6=''){
	write("error", $msg, $msg1, $msg2, $msg3, $msg4, $msg5, $msg6);
}
function debug($msg, $msg1='', $msg2='', $msg3='', $msg4='', $msg5='', $msg6=''){
	write("debug", $msg, $msg1, $msg2, $msg3, $msg4, $msg5, $msg6);
}
function set_log_level($_level){
	global $log_level;
	global $log_level_prev;
	//$log_level_prev = $log_level; commenting this - restore will always restore the original (from sett file) level
	$log_level = $_level;
}
// restores previous log level; used when it's necessary to change log level for a piece of code and then restore previous value
function restore_log_level(){
	global $log_level;
	global $log_level_prev;
	$log_level = $log_level_prev;
}
?>
